/*
 * Copyright (c) 2004-2009, Luiz Otavio O Souza <loos.br@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * $Id: xml-data.h 96 2009-01-20 15:34:19Z loos-br $
 */

#ifndef XML_DATA_H
#define XML_DATA_H

#include "string.h"
#include "queue.h"

struct xml_tag_opt_ {

    SLIST_ENTRY(xml_tag_opt_)	xml_tag_opt__;

    string		name;
    string		value;
};

struct xml_tag_opts {                                                           \
    struct xml_tag_opt_	*slh_first;		/* first element */                     \
};

struct xml_tag_ {

    SLIST_ENTRY(xml_tag_)	xml_tag__;

    string		name;			/* tag name */
    string		data;			/* tag data */
    struct xml_tags {
	struct xml_tag_	*slh_first;		/* first element */
    }			xml_tag_head;
    struct xml_tag_opts	xml_tag_opt_head;	/* tag options */
};

#endif
