<?php
/*
 * Copyright (c) 2004-2009 Luiz Otavio O Souza <loos.br@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * $Id: default.array.cmd.inc.php 102 2009-02-05 18:37:40Z loos-br $
 */

    $arrayCMD = array(

	"CVR0"			=> 0x00000001,
	"MSNP8"			=> 0x00000002,
	"MSNP9"			=> 0x00000004,
	"MSNP10"		=> 0x00000008,
	"MSNP11"		=> 0x00000010,
	"MSNP12"		=> 0x00000020,
	"MSNP13"		=> 0x00000040,
	"MSNP14"		=> 0x00000080,
	"MSNP15"		=> 0x00000100,
	"MSNP16"		=> 0x00000200,
	"MSNP17"		=> 0x00000400,
	"MSNP18"		=> 0x00000800,
	"UNKNOWN"		=> 0xF0000FFF,

	"USEIMG"		=> 0x00001000,
	"SEEIMG"		=> 0x00002000,
	"FILETRANSF"		=> 0x00004000,
	"CHAT"			=> 0x00008000,
	"P2P"			=> 0x00010000,
	"DATACAST"		=> 0x00020000,
	"CONNECT"		=> 0x00040000,
	"SAVE_MSG"		=> 0x00080000,
	"SAVE_CONTACTS"		=> 0x00100000,
	"CONTACT_DENY"		=> 0x00200000,
	"CONTACT_BLOCKED"	=> 0x00400000,
	"EMOTICON"		=> 0x00800000,
	"LOGWARNING"		=> 0x01000000,
	"CONTACT_REMOVED"	=> 0x02000000,

	);
?>
