<?php
/*
 * Copyright (c) 2004-2009 Luiz Otavio O Souza <loos.br@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * $Id: default.array.option.inc.php 102 2009-02-05 18:37:40Z loos-br $
 */

    $arrayOption = array(

	array ( "name" => "msnp8", "cmd" => "MSNP8",
		"desc" => "Bloquear o uso do protocolo MSNP8 (Windows Messenger, aMSN, Gaim, etc.)" ),

	array ( "name" => "msnp12", "cmd" => "MSNP12",
		"desc" => "Bloquear o uso do protocolo MSNP12 (Gaim?, Messenger 7, 7.5)" ),

	array ( "name" => "msnp13", "cmd" => "MSNP13",
		"desc" => "Bloquear o uso do protocolo MSNP13 (Live Messenger 8.0)" ),

	array ( "name" => "msnp15", "cmd" => "MSNP15",
		"desc" => "Bloquear o uso do protocolo MSNP15 (Live Messenger 8.x)" ),

	array ( "name" => "msnp18", "cmd" => "MSNP18",
		"desc" => "Bloquear o uso do protocolo MSNP18 (Live Messenger 9.x)" ),

	array ( "name" => "filetransf", "cmd" => "FILETRANSF",
		"desc" => "Bloquear a transferência de arquivos" ),

	array ( "name" => "chat", "cmd" => "CHAT",
		"desc" => "Bloquear Chat (troca de mensagens de texto)" ),

	array ( "name" => "p2p", "cmd" => "P2P",
		"desc" => "Bloquear P2P (troca de imagens e arquivos)" ),

	array ( "name" => "datacast", "cmd" => "DATACAST",
		"desc" => "Bloquear datacast (Winks, pedidos de atenção)" ),

	array ( "name" => "emoticon", "cmd" => "EMOTICON",
		"desc" => "Bloquear emoticons animados" ),

	array ( "name" => "useimg", "cmd" => "USEIMG",
		"desc" => "Bloquear o uso de imagem do contato" ),

	array ( "name" => "seeimg", "cmd" => "SEEIMG",
		"desc" => "Bloquear a visualiza&ccedil;&atilde;o de imagens dos contatos" ),

	array ( "name" => "logwarning", "cmd" => "LOGWARNING",
		"desc" => "Avisa ao usu&aacute;rio que a mensagem esta sendo monitorada" ),

	array ( "name" => "contactdeny", "cmd" => "CONTACT_DENY",
		"desc" => "Bloquear novos usuários" ),

	NULL

	);

?>
