<?php
/*
 * Copyright (c) 2004-2009 Luiz Otavio O Souza <loos.br@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * $Id: class.ctl.php 102 2009-02-05 18:37:40Z loos-br $
 */

    class ctl {
	var $sock	= NULL;

	function connect($debug = 0) {
	    if ($debug) {
		$this->sock = socket_create(AF_UNIX, SOCK_STREAM, 0);
		socket_set_nonblock($this->sock);
		socket_connect($this->sock, "/var/run/msn-proxy.ctl");
	    } else {
		$this->sock = @socket_create(AF_UNIX, SOCK_STREAM, SOL_TCP);
		@socket_set_nonblock($this->sock);
		@socket_connect($this->sock, "/var/run/msn-proxy.ctl");
	    }
	}

	function write($buf = NULL, $s = 0) {
	    $size = $s;
	    if ($size == 0)
		$size = strlen($buf);

	    while ($size > 0) {
		if (($len = socket_write($this->sock, $buf, $size)) === false)
		    return true;
		$size -= $len;
	    }
	    return false;
	}

	function close() {
	    socket_close($this->sock);
	}
    }
?>
