<?php
/*
 * Copyright (c) 2004-2009 Luiz Otavio O Souza <loos.br@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * $Id: default.array.status.inc.php 102 2009-02-05 18:37:40Z loos-br $
 */

    $arrayStatus = array(

	"BSY" => array ( "img" => "ocupado.gif", "msg" => "Ocupado"),
	"NLN" => array ( "img" => "online.gif", "msg" => "Online"),
	"AWY" => array ( "img" => "ausente.gif", "msg" => "Ausente"),
	"PHN" => array ( "img" => "ocupado.gif", "msg" => "Ao telefone"),
	"HDN" => array ( "img" => "offline.gif", "msg" => "Invisível"),
	"BRB" => array ( "img" => "ausente.gif", "msg" => "Volto já"),
	"LUN" => array ( "img" => "ausente.gif", "msg" => "Em horário de almoço"),
	"IDL" => array ( "img" => "ausente.gif", "msg" => "Inativo"),
	"OFF" => array ( "img" => "offline.gif", "msg" => "Offline")

	);
?>
