<?php
/*
 * Copyright (c) 2004-2009 Luiz Otavio O Souza <loos.br@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * $Id: header.inc.php 102 2009-02-05 18:37:40Z loos-br $
 */

    // Date in the past
    header("Expires: Mon, 26 Jul 1997 05:00:00 GMT");

    // always modified
    header("Last-Modified: " . gmdate("D, d M Y H:i:s") . " GMT");

    // HTTP/1.1
    header("Cache-Control: no-store, no-cache, must-revalidate");
    header("Cache-Control: post-check=0, pre-check=0", false);

    // HTTP/1.0
    header("Pragma: no-cache");

    header("Content-Type: text/html; charset=iso-8859-1");

    session_start();

?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<HTML>
<HEAD>
<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=iso-8859-1">
<TITLE>msn-proxy</TITLE>
<style type="text/css">
a:hover {
    text-decoration: underline;
    color: #000000;
}
a {
    text-decoration: none;
    color: #000000;
}
BODY {
    text-indent: 0;
    font-family             :Verdana, Arial;
    font-size               :9px;
}
td {
    font-family             :Verdana, Arial;
    font-size               :9px;
}
INPUT {
    font-family             :Verdana;
    font-size               :10px;
}
SELECT {
    font-family             :Verdana;
    font-size               :10px;
}
table tr.odd {
    background-color	    :#F5F5F5;
}
table tr.even {
    background-color	    :#EBEBEB;
}
table tr.odd:hover,
table tr.even:hover {
    background-color	    :#FFFFFF;
}
.odd:hover,
.even:hover,
.hover {
    background-color	    :#FFFFFF;
}
</style>
</HEAD>
