/*
 * Copyright (c) 2004-2009, Luiz Otavio O Souza <loos.br@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * $Id: return.h 96 2009-01-20 15:34:19Z loos-br $
 */

#ifndef RETURN_H
#define RETURN_H

#define	ROK		0x0001
#define RNO		0x0002
#define REND		0x0003
#define RBUSY		0x0004
#define RFAIL		0x0005
#define RSKIP		0x0006
#define RCLOSE		0x0007
#define RTIMEOUT	0x0008
#define RCONTINUE	0x0009
#define RNOSESSION	0x000A
#define RDISCONNECT	0x000B
#define RINVALIDADDR	0x0100

#define	RNOMEM		0x1000
#define RNOCGI		0x2000
#define	ECGIENV		0x3000

#endif
