/*
 * Copyright (c) 2004-2009, Luiz Otavio O Souza <loos.br@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * $Id: net-io.h 96 2009-01-20 15:34:19Z loos-br $
 */

#ifndef NET_IO_H
#define NET_IO_H

#include <sys/types.h>
#include <sys/socket.h>

#include "config.h"

int	set_options(int fd);
int	set_nonblock(int fd);
int	unix_listen(int backlog);
int	net_listen(const char *host, const char *port, int backlog);
int	net_connect(const char *host, const char *port);
string	*bind_free_port( unsigned int port_min,
                        unsigned int port_max,
                        string *host,
                        int *listenfd);
int	resolve_client(string *str_host, struct sockaddr *sa, socklen_t sa_len);

#endif
