/*
 * Copyright (c) 2004-2009, Luiz Otavio O Souza <loos.br@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * $Id: array_cmd.h 96 2009-01-20 15:34:19Z loos-br $
 */

#ifndef ARRAY_CMD_H
#define ARRAY_CMD_H

/*
 * the first three bytes are reserved for client version
 */

#define USEIMG		0x00001000
#define SEEIMG		0x00002000
#define FILETRANSF	0x00004000
#define CHAT		0x00008000
#define P2P		0x00010000
#define DATACAST	0x00020000
#define CONNECT		0x00040000
#define SAVE_MSG	0x00080000
#define SAVE_CONTACTS	0x00100000
#define	CONTACT_DENY	0x00200000
#define	CONTACT_BLOCKED	0x00400000
#define	EMOTICON	0x00800000
#define	LOGWARNING	0x01000000
#define	CONTACT_REMOVED	0x02000000

#define DENY_ALL	0xffffffff

#endif
