/*
 * Copyright (c) 2004-2009, Luiz Otavio O Souza <loos.br@gmail.com>
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

/*
 * $Id: fmt.h 96 2009-01-20 15:34:19Z loos-br $
 */

#ifndef FMT_H
#define FMT_H

#include <sys/types.h>
#include <stdarg.h>

unsigned int fmt_str(register unsigned char *s, register const char *t);
unsigned int fmt_sql(register unsigned char *s, register const char *t);
unsigned int fmt_ulong(register unsigned char *s, register unsigned long u);
unsigned int fmt_u64(register unsigned char *s, register __uint64_t);
unsigned int fmt_vprintf(char unsigned *s, const char *fmt, va_list ap);
unsigned int fmt_printf(char unsigned *s, const char *fmt, ...);

#endif
